package shukan;


/**
 * Module
 * Modified by Chevalier Noemy, Berrada Louise and Piquerez Corentin
 */
public class ShukanModule
{
  /** Module name */
  private String name;
  /** Module teachers */
  private String subtitle = null;
  /** Queue of module activities */
  private int[] activ = null;
  /** Queues of planed activities */
  private ShukanSchedule sched = null;
  /** Module scheduling status */
  private boolean scheduled = false;
  /** Teacher affectation for main courses */
  private int[] affectCM = null;
  /** Teacher affectation for supervised trainings */
  private int[] affectTD = null;
  /** Teacher affectation for practical works */
  private int[] affectTP = null;
  /** Teacher affectation for each activity */
  private int[][] affect = null;
  /** Left shift status for activities */
  private boolean[] onLeft = null;
  /** Right shift status for activities */
  private boolean[] onRight = null;
  /** Working weeks for the module */
  private int[] activeWeeks = null;

  /** Specific activity : main course */
  public final static int ACT_CM = 0;   // Cours magistral
  /** Specific activity : 1 hour long control */
  public final static int ACT_EV2 = 1;  // Evaluation d'1h
  /** Specific activity : 2 hours long control */
  public final static int ACT_EV1 = 2;  // Evaluation de 2h
  /** Specific activity : supervised training in standard room */
  public final static int ACT_TD = 3;   // TD en salle de cours
  /** Specific activity : supervised training in computer room */
  public final static int ACT_TM = 4;   // TD en salle ordi
  /** Specific activity : supervised training in parallel */
  public final static int ACT_TV = 5;   // TD en parallele (eval)
  /** Specific activity : SAe supervised training in standard room */
  public final static int ACT_SD = 6;   // TD de SAE en salle de cours
  /** Specific activity : SAe supervised training in computer room */
  public final static int ACT_SM = 7;   // TD de SAE en salle ordi
  /** Specific activity : Practical work in computer room */
  public final static int ACT_TP = 8;   // TP en salle ordi
  /** Specific activity : 4 hours long practical work in computer room */
  public final static int ACT_TP4 = 8;   // TP de 4h en salle ordi
  /** Specific activity : 2 hours long practical work in computer room */
  public final static int ACT_TP2 = 9;   // TP de 2h en salle ordi
  /** Specific activity : 2 hours long practical work in standard room */
  public final static int ACT_TQ2 = 10;  // TP en salle de cours
  /** Specific activity : SAe practical work in computer room */
  public final static int ACT_SP = 11;  // TP de SAE en salle ordi
  /** Specific activity : SAe practical work in standard room */
  public final static int ACT_SQ = 12;  // TQ de SAE en salle de cours
  /** Activity main types */
  private final static String[] MAIN_TYPES = {"CM", "EVAL", "EVAL",
                                            "TD", "TD", "TD en //", "TD SAE", "TD SAE",
                                            "TP", "TP", "TP", "TP SAE", "TP SAE"};
  /** Activity rooms */
  private final static String[] ROOMS = {"Amphi/Cours", "Amphi/Cours", "Amphi/Cours",
                                            "Cours", "Machine", "Cours", "Cours", "Machine",
                                            "Machine", "Machine", "Cours", "Machine", "Cours"};
  /** Activity types */
  public final static String[] TYPES = {"CM", "EV2", "EV1",
                                         "TD", "TM", "TV", "SD", "SM",
                                         "TP4", "TP2", "TQ2", "SP", "SQ"};
  /** Activity symbol */
  public final static String[] DISPLAY = {"C", "E", "e",
                                          "d", "m", "v", "s", "z",
                                          "P", "p", "q", "S", "Z"};
  /** Activity duration (in hours) */
  public final static int[] SLOAD = {2, 2, 1, 2, 2, 2, 2, 2, 4, 2, 2, 2, 2};

  /** Default maximal number of activities in a module */
  private final static int DEFAULT_ACTIVITY_NUMBER = 30;
  /** Maximal number of activities per week */
  public final static int MAX_ACTIV_PER_WEEK = 6;


  /** Creation of a module */
  public ShukanModule (String name)
  {
    this.name = new String (name);
    activ = new int[DEFAULT_ACTIVITY_NUMBER];
    affect = new int[DEFAULT_ACTIVITY_NUMBER][];
  }

  /** Adds an activity */
  public boolean addActivity (int num, String type)
  {
    int numT = -1;
    for (int i = 0; i < TYPES.length && numT == -1; i++)
      if (type.equals (TYPES[i])) numT = i;
    if (numT == -1) return (false);
    if (num == activ.length)
    {
      int[] tmp = new int[activ.length + 1];
      int[][] tmpaff = new int[activ.length + 1][];
      for (int i = 0; i < activ.length; i++)
      {
        tmp[i] = activ[i];
        tmpaff[i] = affect[i];
      }
      activ = tmp;
      affect = tmpaff;
    }
    activ[num] = numT;
    affect[num] = new int[0];
    return (true);
  }

  /** Adds a new activity */
  public boolean addNewActivity (int num, String type)
  {
    int numT = -1;
    int j;
    for (int i = 0; i < TYPES.length && numT == -1; i++)
      if (type.equals (TYPES[i])) numT = i;
    if (numT == -1) return (false);

    int[] tmp = new int[activ.length + 1];
    int[][] tmpaff = new int[activ.length + 1][];
    for (int i = 0; i < activ.length+1; i++)
    {
      if (i!=num){
        j = i;
        if (i>num) j--;
        tmp[i] = activ[j];
        tmpaff[i] = affect[j];
      }
    }

    activ = tmp;
    affect = tmpaff;
    activ[num] = numT;
    affect[num] = new int[0];
    return (true);
  }

  /** Adjusts the activity array.
   * @param size Number of activities to be planned.
   */
  public void closeActivities (int size)
  {
    if (size != activ.length)
    {
      int[] tmp = new int[size];
      int[][] tmpaff = new int[size][];
      for (int i = 0; i < size; i++)
      {
        tmp[i] = activ[i];
        tmpaff[i] = affect[i];
      }
      activ = tmp;
      affect = tmpaff;
    }

    onLeft = new boolean[activ.length];
    onRight = new boolean[activ.length];
    for (int i = 0; i < onLeft.length; i++)
    {
      onLeft[i] = false;
      onRight[i] = false;
    }
  }

  /** deletes an activity */
  public void deleteActivity (int index)
  {

    int size = activ.length-1;
    int[] tmp = new int[size];
    int[][] tmpaff = new int[size][];
    boolean[] tmponLeft = new boolean[activ.length-1];
    boolean[] tmponRight = new boolean[activ.length-1];
    for (int i = 0; i < index; i++)
    {
      tmp[i] = activ[i];
      tmpaff[i] = affect[i];
      tmponLeft[i] = onLeft[i];
      tmponRight[i] = onRight[i];
    }
    for (int i = index+1; i < size+1; i++)
    {
      tmp[i-1] = activ[i];
      tmpaff[i-1] = affect[i];
      tmponLeft[i-1] = onLeft[i];
      tmponRight[i-1] = onRight[i];
    }

    activ = tmp;
    affect = tmpaff;
    onLeft = tmponLeft;
    onRight = tmponRight;


  }
  /** Automatically plans a free module
   */
  public void autoPlan ()
  {
    if (sched == null)
    {
      sched = new ShukanSchedule (activeWeeks.length, MAX_ACTIV_PER_WEEK);
      for (int i = 0; i < activeWeeks.length; i++)
        if (activeWeeks[i] == 0) sched.kill (i);
    }
    int durall = 0;
    for (int i = 0; i < activeWeeks.length; i++) durall += activeWeeks[i];
    double period = durall / (float) (activ.length);
    int remainder = 0;
    int placed = 0;
    int lastWeek = 0;
    for (int i = 0; i < activeWeeks.length && placed < activ.length; i++)
    {
      remainder += activeWeeks[i];
      while (remainder > period / 2 && placed < activ.length)
      {
        sched.add (i, placed++);
        lastWeek = i;
        remainder -= period;
      }
    }
    while (placed < activ.length) sched.add (lastWeek, placed++);
    scheduled = true;
  }

  /** Schedules all activities */
  public void schedule (int[] weeks)
  {
    if (sched == null)
    {
      sched = new ShukanSchedule (activeWeeks.length, MAX_ACTIV_PER_WEEK);
      for (int i = 0; i < activeWeeks.length; i++)
        if (activeWeeks[i] == 0) sched.kill (i);
    }
    for (int i = 0; i < weeks.length; i++) sched.add (weeks[i], i);
    scheduled = true;
  }

  /** Unschedules all the module */
  public void unschedule ()
  {
    sched.clear ();
    scheduled = false;
  }

  /** Unschedules an activity of the module */
  public void unschedule (int index, int week, int nb_week)
  {
    sched.delete(index, week, nb_week);
  }

  /** Schedules an activity */
  public void schedule (int num, int week)
  {
    if (sched == null)
    {
      sched = new ShukanSchedule (activeWeeks.length, MAX_ACTIV_PER_WEEK);
      for (int i = 0; i < activeWeeks.length; i++)
        if (activeWeeks[i] == 0) sched.kill (i);
    }
    sched.add (week, num);
    scheduled = true;
  }

  /** Returns the schedule status */
  public boolean isScheduled ()
  {
    return (scheduled);
  }

  /** Returns the name of the module */
  public String name ()
  {
    return (name);
  }

  /** Sets the module start week */
  public void setStartWeek (int num)
  {
    for (int i = 0; i < num; i++) disableWeek (i);
  }

  /** Sets the module end week */
  public void setEndWeek (int num)
  {
    for (int i = num + 1; i < activeWeeks.length; i++) disableWeek (i);
  }

  /** Sets a module holly week */
  public void setHollyWeek (int num)
  {
    disableWeek (num);
  }

  /** Returns the activities */
  public int[] activities ()
  {
    return (activ);
  }

  /** Returns the activity schedule */
  public int[] schedule (int week)
  {
    return (sched.contents (week));
  }

  /** Returns the complete activity schedule */
  public int[] schedule ()
  {
    return (sched.contents ());
  }

  /** Shift the selection leftwards */
  public int left (int week, int ln)
  {
    return ((ln == 1) ? sched.left (week) : sched.left (week, week + ln - 1));
  }

  /** Shifts the selection rightwards */
  public int right (int week, int ln)
  {
    return ((ln == 1) ? sched.right (week) : sched.right (week, week + ln - 1));
  }

  /** Returns the student load for the given week */
  public int studentLoad (int week)
  {
    int[] sch = sched.contents (week);
    int load = 0;
    for (int i = 0; i < sch.length; i++) load += SLOAD[activ[sch[i]]];
    return (load);
  }

  /** Returns a teacher load for the given week */
  public int teacherLoad (int teacher, int week)
  {
    int[] sch = sched.contents (week);
    int load = 0;
    for (int i = 0; i < sch.length; i++)
      for (int j = 0; j < affect[sch[i]].length; j++)
        if (teacher == affect[sch[i]][j]) load += SLOAD[activ[sch[i]]];
    return (load);
  }

  /** Affects activities to teachers */
  public void setTeachers (int act, int[] teacher)
  {
    if (act == ACT_TP)
    {
      affectTP = teacher;
      for (int i = 0; i < activ.length; i++)
        if (activ[i] >= act) affect[i] = teacher;
    }
    else if (act == ACT_TD)
    {
      affectTD = teacher;
      for (int i = 0; i < activ.length; i++)
        if (activ[i] >= act && activ[i] < ACT_TP) affect[i] = teacher;
    }
    else
    {
      affectCM = teacher;
      for (int i = 0; i < activ.length; i++)
        if (activ[i] < ACT_TD) affect[i] = teacher;
    }
  }

  /** Affects specific activity to teachers */
  public void affectSpec (int act, int[] teacher)
  {
    affect[act] = teacher;
  }

  /** Sets an activity to be displayed on left */
  public void setOnLeft (int act)
  {
    onRight[act] = false;
    onLeft[act] = true;
  }
  /** Sets an activity to be displayed on middle */
  public void setOnMiddle (int act, boolean ajout) {
    if (ajout) {
      boolean[] tmpRight = new boolean[onLeft.length + 1];
      boolean[] tmpLeft = new boolean[onLeft.length + 1];
      for (int i = 0; i < act; i++) {
        tmpLeft[i] = onLeft[i];
        tmpRight[i] = onRight[i];
      }
      tmpRight[act] = false;
      tmpLeft[act] = false;
      for (int i = act+1; i < onLeft.length+1; i++) {
        tmpLeft[i] = onLeft[i-1];
        tmpRight[i] = onRight[i-1];
      }
      onLeft = tmpLeft;
      onRight = tmpRight;
    } else {
      onLeft[act] = false;
      onRight[act] = false;
    }

  }
  /** Sets an activity to be displayed on right */
  public void setOnRight (int act)
  {
    onLeft[act] = false;
    onRight[act] = true;
  }

  /** Returns the activities to be displayed on left */
  public boolean[] activitiesOnLeft ()
  {
    return (onLeft);
  }

  /** Returns the activities to be displayed on right */
  public boolean[] activitiesOnRight ()
  {
    return (onRight);
  }

  /** Sets a subtitle to the module */
  public void setSubtitle (String text)
  {
    subtitle = text;
  }

  /** Returns the subtitle (null by default) */
  public String subtitle ()
  {
    return (subtitle);
  }

  /** Defines the module active weeks. */
  public void setActiveWeeks (int[] aw)
  {
    activeWeeks = aw;
  }

  /** Disables a week activation */
  public void disableWeek (int num)
  {
    activeWeeks[num] = 0;
  }

  /** Returns the module active weeks. */
  public int[] activeWeeks ()
  {
    return (activeWeeks);
  }

  /** Returns the module progression. */
  public String[] progression ()
  {
    String[] prog = new String[activ.length];
    for (int i = 0; i < prog.length; i++) prog[i] = TYPES[activ[i]];
    return (prog);
  }

  /** Returns the teachers for each kind of activities. */
  public int[] teachers (int act)
  {
    if (act == ACT_TP) return (affectTP);
    else if (act == ACT_TD) return (affectTD);
    else return (affectCM);
  }

  /** Returns the specific teachers for all activities. */
  public int[][] specTeachers ()
  {
    int[][] st = new int[activ.length][];
    for (int i = 0; i < activ.length; i++)
    {
      int[] aff = affectCM;
      if (activ[i] >= ACT_TP) aff = affectTP;
      else if (activ[i] >= ACT_TD) aff = affectTD;

      boolean ok = true;
      if (affect[i].length != aff.length) ok = false;
      else
      {
        for (int j = 0; ok && j < affect[i].length; j++)
          if (affect[i][j] != aff[j]) ok = false;
      }
      st[i] = (ok ? new int[0] : affect[i]);
    }
    return (st);
  }

  /** Returns the set-on-left activities. */
  public boolean[] onLeftActivities ()
  {
    return (onLeft);
  }

  /** Returns the set-on-right activities. */
  public boolean[] onRightActivities ()
  {
    return (onRight);
  }

  /** Returns the count of activities. */
  public int activityCount ()
  {
    return (activ == null ? 0 : activ.length);
  }

  /** Returns the week and activity at given position. */
  public int select (int position)
  {
    return (sched.select (position));
  }

  /** Converts the module in a string */
  public String toString ()
  {
    String s = name + " " + activ.length;
    for (int i = 0; i < activ.length; i++)
      s += " " + TYPES[activ[i]];
    return s;
  }

  /** Exchanges the activities at given slots */
  public void exchange (int a1, int a2)
  {
    int n1 = sched.identify (a1);
    int n2 = sched.identify (a2);
    int val = activ[n1];
    activ[n1] = activ[n2];
    activ[n2] = val;
    int[] aff = affect[n1];
    affect[n1] = affect[n2];
    affect[n2] = aff;
    boolean on = onLeft[n1];
    onLeft[n1] = onLeft[n2];
    onLeft[n2] = on;
    on = onRight[n1];
    onRight[n1] = onRight[n2];
    onRight[n2] = on;
  }

  /** convert a number of activity into a String */
  public String activityName(int number){
    return TYPES[number];
  }
  /** convert a number of activity into a String */
  public String activityMainName(int number){
    return MAIN_TYPES[number];
  }

  /** returns the duration of an activity */
  public int activityDuration(int number){
    return SLOAD[number];
  }

  /** returns the room of an activity */
  public String activityRoom(int number){
    return ROOMS[number];
  }
}
